#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 05-Jun-2003
#################################################################################
#
# Gummies  - Script part of Ethereal Realms, designed to view gummies 
#            rights.  Allows to display specific classes and groups.
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Param;						# Parameter handler
use eRealms::Table;						# Table control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter handle


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameters
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Display gummy
	GummyDisplay($database, $cgi, $param);

	# Document footer
	$database->DocumentGetFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Gummy Display
#
# Very simple sub-routine that will simply allow the user to look through the 
# list of gummies to search for.

sub GummyDisplay
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle

	my $res;						# Results			
	my $tmpl;						# Template handle
	my $statement;						# Database statement

	my $class;						# Gummy class
	my $count;						# Counter
	my $ident;						# Gummy indentier
	my $title;						# Display title

	my $fst_name;						# First stored name
	my $fst_path;						# First stored path
	my $sec_name;						# Second stored name
	my $sec_path;						# Second stored path

	my $group;						# Gummy grouping 
	my $realm    = 'system';				# Realm search

	my $widget;
	

	my @classes;						# Gummy classes
	my %config;						# Configuration hash


	#####################
	# Program area

	# Link with has
	$database->GetHashConfiguration(\%config);

	# Create instance
	$tmpl = new eRealms::Template(\%config);


	# Widget generation
	# Pull list of gummies
	$database->Pull(\$statement, "SELECT GummyClass, GummyIdent
		FROM Gummy
		WHERE GummyClass <> 'default'
		GROUP BY GummyIdent, GummyClass
		ORDER BY GummyClass, GummyIdent"); 

	# Set
	$ident = undef;

	# Populate
	while (($class, $group) = $statement->fetchrow())
	{
		# Global select
		$ident = $class if (defined($group));

		# Implement if end of list
		if ((!defined($group))
		 && (defined($ident)))
		{
			# Class define
			$ident = ucfirst($ident) . ' - ' . $config{'TxtGlobal'};

			# Add to list
			push(@classes, $ident);


			# Reset
			$ident = undef;
		}


		# Build string
		$class = (defined($group)) ? ucfirst($class) . ' - ' . ucfirst($group) : ucfirst($class);

		# Pupulate
		push(@classes, $class);
	}

	# Finish
	$statement->finish();


	# Default is general
	$class = (defined($param->{'LIST'})) ? $param->{'LIST'} : 'General';


	# Widget
	$widget = $cgi->popup_menu('LIST', \@classes, $class);



	# Information display
	# Display title
	$tmpl->ShowMain($config{TagGummies});

	# Table Top
	print "<CENTER>\n<TABLE WIDTH=500>\n <TR>\n  <TD COLSPAN=7>\n";

	print $cgi->start_form();
	print $tmpl->Pass('TmplList', 
		MLIST   => $widget,
		MSUBMIT => $cgi->submit($config{'TxtSubmitList'})
	 );
	print $cgi->end_form(), "\n";

        print "  </TD>\n </TR>\n";


	# Classes
	if (defined($class))
	{
		# Divy up
		($class, $group) = split(/\s-\s/, $class);


		# Query management
		# Prepare and execute
		if (defined($group))
		{
			# Specific
			if ($group ne $config{'TxtGlobal'})
			{
				# Group specific
				$database->Pull(\$statement, "SELECT 
					 GummyName  AS \"GummyName\",
					 GummyPath  AS \"GummyPath\",
					 GummyIdent AS \"GummyIdent\"
					FROM  Gummy
					WHERE LOWER(GummyClass)=?
					  AND LOWER(RealmName)=?
					  AND LOWER(GummyIdent)=?
					ORDER BY GummyIdent, GummyName",

				 lc($class),
				 lc($realm),
				 lc($group)
				);

				# Set group
				$group = lc($group);
			}

			# Global
			else
			{
				# General search
				$database->Pull(\$statement, "SELECT 
					 GummyName  AS \"GummyName\",
					 GummyPath  AS \"GummyPath\",
					 GummyIdent AS \"GummyIdent\"
					FROM  Gummy
					WHERE LOWER(GummyClass)=?
					  AND LOWER(RealmName)=?
					ORDER BY GummyIdent, GummyName",

				 lc($class),
				 lc($realm)
				);

				# Set group
				$group = 'None';
			}
		}

		else
		{
			# Non grouped
			$database->Pull(\$statement, "SELECT 
				 GummyName  AS \"GummyName\",
				 GummyPath  AS \"GummyPath\",
				 GummyIdent AS \"GummyIdent\"
				FROM  Gummy
				WHERE LOWER(GummyClass)=?
				  AND LOWER(RealmName)=?
				  AND GummyIdent IS NULL
				ORDER BY GummyIdent, GummyName",

			 lc($class),
			 lc($realm)
			);

			# Set group
			$group = 'None';
		}



		# Title
		# Determine String
		$title = (defined($group)) ? ucfirst($class) . ' - ' . ucfirst($group) : ucfirst($class);

		# Generate Title
		$title = $tmpl->PassTitle($title);


		# Reset values
		$count = 0;


		# Display title
		print " <TR>\n",
		      "  <TD COLSPAN=7 HEIGHT=15>&nbsp;</TD>\n",
		      " </TR>\n";
		print " <TR>\n",
		      "  <TD COLSPAN=7>$title</TD>\n",
		      " </TR>\n";


		# Cycle through gummies
		while ($res = $statement->fetchrow_hashref())
		{
			# Increment counter
			$count++;


			# Padding
			$ident = (defined($res->{'GummyIdent'})) ? lc($res->{'GummyIdent'}) : 'None';

			# Title
			if ($ident ne $group)
			{
				# Display single if need be
				if (($count % 2) == 0)
				{
					# Display to screen
					print " <TR>\n",
					      "  <TD WIDTH=10>&nbsp;</TD>\n",
					      "  <TD CLASS=\"ITEMTITLE\">$fst_name</TD>\n",
					      "  <TD><IMG SRC=\"$fst_path\"></TD>\n",
					      "  <TD WIDTH=50 COLSPAN=3>&nbsp;</TD>\n",
					      "  <TD WIDTH=10>&nbsp;</TD>\n",
					      " </TR>\n";
				}	

				# Reset counter
				$count = 1;


				# Prepare for new title
				# Alternate
				$group = $ident;

				# Uppercase
				$ident = ucfirst($ident);

				# Generate title
				$title = $tmpl->PassSub($ident);


				# Display title
				print " <TR>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      "  <TD COLSPAN=5 HEIGHT=15>&nbsp;</TD>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      " </TR>\n";
				print " <TR>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      "  <TD COLSPAN=5>$title</TD>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      " </TR>\n";
			}


			# Store for next pass
			if (($count % 2) != 0)
			{
				# Assign values
				$fst_name = $res->{GummyName};
				$fst_path = $res->{GummyPath};
			}

			# Display row
			else
			{
				# Assign values
				$sec_name = $res->{GummyName};
				$sec_path = $res->{GummyPath};
			

				# Display to screen
				print " <TR>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      "  <TD CLASS=\"ITEMTITLE\">$fst_name</TD>\n",
				      "  <TD><IMG SRC=\"$fst_path\"></TD>\n",
				      "  <TD WIDTH=50>&nbsp;</TD>\n",
				      "  <TD>$sec_name</TD>\n",
				      "  <TD CLASS=\"ITEMTITLE\"><IMG SRC=\"$sec_path\"></TD>\n",
				      "  <TD WIDTH=10>&nbsp;</TD>\n",
				      " </TR>\n";
			}
		}
	}

	# End table
	print "</TABLE>\n<CENTER>\n";
}
