################################################################################
# Created       : Martin Foster
# Modified      : 18-Sep-2003
################################################################################
#
# Comm - Module part of Ethereal Realms, designed to prompt and handle posts.
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package eRealms::Comm;						# Pacakace name

use eRealms::Filter;						# Filter system
use eRealms::Option;						# Options handling
use eRealms::Stamp;						# Timestamp handler


################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle

	my $self     = {
		HANDLE  => $database,
		CGI     => $cgi
 	 };


	#####################
	# Program Area

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}



#####################
# Comm
#
# Subroutine primarily charged with displaying input area.   This will also call
# CommHandler which will handle the requests to determin the need for a new
# database entry

sub Comm
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $choice   = shift;					# Choice of the matter

	my $database = $self->{'HANDLE'};			# Database handle
	my $cgi      = $self->{'CGI'};				# CGI handle

	my $js;							# Javascript header
	my $jscode;						# Javascript code
	my $jslink   = '';					# Javascript link code
	my $jsstrm;						# Javascript stream code
 
	my $header;						# Independant header
	my $panel;						# Communications panel

	my $basic;						# Basic parameter list
	my $inline;						# Inline parameter list
	my $url      = $cgi->url(-absolute=>1);			# URL of script

	my $comm     = $param->{'COMM'};			# Comm value
	my $color;						# Colour default
	my $default;						# Default colour
	my $locate;						# Location
	my $face;						# Font face
	my $puppet;						# Puppet name
	my $size;						# Size default
	my $text;						# Text colour

	my $access;						# Access control
	my $locale;						# Default settings

	my @color;						# Colours
	my @face;						# Font face
	my @locale;						# Locale settings
	my @size;						# Size fonts
	my @char;						# Character

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with Hash
	$database->GetHashRealm(\%realm);

	# Locale setting
	($access) = $database->DataGetRealmAccess($param->{'ROOM'});
	($locale) = $database->DataGetRealmLocale($param->{'ROOM'});


	# Populate arrays
	@color = FontSelectColor($self, $param);
	@size  = FontSelectSize($self, $param);
	@face  = FontSelectFace($self, $param);

	# Sorting
	@color = sort(@color);


	# Pupulate puppet list
	$database->GroupList(\@char, $access, $param->{'USER'}, $param->{'ROOM'});

	# Store character name
	$puppet = $param->{'CHAR'};

	# Parameter cleanup
	$param->Cleanup(
	  'COLOR',
	  'COMM',
	  'FACE',
	  'POST',
	  'SIZE',
	  $realm{'TxtSubmitButton'}
	 );


	# Inline parameter handling
	# Basic inline
	$basic  = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'});

	# Normal use inline
	$inline = $param->EmbedInline($param->Flat());

	# Pull realm default
	($default) = $database->DataGetRealmText($param->{'ROOM'});	

	# Retreive size and color
	($size, $color, $face) = $database->DataGetFont($puppet, $param->{'USER'}, $default, $face[0]);

	# Determine need for javascript
	($js) = $database->DataGetJavascript($param->{'USER'});

	# Independent HTML
	if ($choice != 0)
	{
		# Extract header
		$header = $database->FetchRealmHeader($param->{'ROOM'});

		# Focus handling
		# Extraction and replacement
		if (($js eq 'yes')
		 && ($header =~ /<(BODY.*?)>/si))
		{
			# Safety mechanism
			# Assign
			my $org = $1;
			my $new = $1;

			# Prepare
			$new = quotemeta($new);

			# Replace
			$header =~ s/$new/$org ONLOAD="document.SEND.POST.focus();"/gs;
		}
			

		# Print file and HTML header
		print $cgi->header();
		print $header, "\n";
	}

	# Javascript handling
	# Print if javascript is desired
	if ($js eq 'yes')
	{ 
		# Assign values
		$jscode = $database->FetchRealmJavascript($param->{'ROOM'}); 

		# Generate javascript code
		$jslink = "ONCLICK='window.open(this.href,this.target,settings); return false;'";


		# Internet explorer and excluding basic
		if (($cgi->user_agent('MSIE'))
		 && ($choice != 0))
		{
			# Set variables
			my $url  = $cgi->url(-absolute=>1);
			my $time = substr(time, -5, 4);

			# Pull widget
			$jsstrm = $database->FetchRealmSupp($param->{'ROOM'});


			# Generate links
			if ((defined($comm))
			 && ($comm =~ /false/i))
			{
				# Attach link
				$jsstrm =~ s/LATTACH/HREF=\"$url$inline&COMM=True\" ACCESSKEY=\"a\" ONCLICK=\"AttachFrames(this.href); return false;\"/gs;
			}
			else
			{
				# Target time 
				my $time = time;

				# Detach link
				$jsstrm =~ s/LATTACH/HREF=\"$url$inline&COMM=False\" TARGET=\"$time\" ACCESSKEY=\"d\" ONCLICK=\"DetachFrames(this.href,this.target); return false;\"/gs;
			}
		}
		# Display code
		print "$jscode\n";
	}

	# Retreive Communications Panel
	$panel = $database->FetchRealmPanel($param->{'ROOM'});


	# Replace links
	# Consisent
	$panel =~ s/LPORTAL/HREF=\"$realm{'LnkPortal'}$basic\" TARGET=\"_BLANK\" ACCESSKEY=\"h\"/gs;

	# Standard links
 	$panel =~ s/LBOARDS/HREF=\"$realm{LnkBoard}$inline\"   TARGET=\"REALMS\" TABINDEX=\"8\" $jslink/gs;
 	$panel =~ s/LIGNORE/HREF=\"$realm{LnkIgnore}$inline\"  TARGET=\"REALMS\" TABINDEX=\"9\" $jslink/gs;
 	$panel =~ s/LIMAILS/HREF=\"$realm{LnkMail}$basic\"     TARGET=\"REALMS\" TABINDEX=\"10\" $jslink/gs;

	# Disjoined
 	$panel =~ s/LCONFIG/HREF=\"$realm{LnkPuppet}$inline\"  TARGET=\"CONFIG\" TABINDEX=\"4\" $jslink/gs;
 	$panel =~ s/LPRIVTE/HREF=\"$realm{LnkPrivate}$inline\" TARGET=\"PRIVTE\" TABINDEX=\"5\" $jslink/gs;
 	$panel =~ s/LDIEROL/HREF=\"$realm{LnkRoll}$inline\"    TARGET=\"DIEROL\" TABINDEX=\"6\" $jslink/gs;
 	$panel =~ s/LWHOSON/HREF=\"$realm{LnkWho}$inline\"     TARGET=\"WHOSON\" TABINDEX=\"7\" $jslink/gs;

	# Helper links
 	$panel =~ s/LCOLORS/HREF=\"$realm{LnkColors}$inline\"  TARGET=\"REALMS\" ACCESSKEY=\"g\" $jslink/gs;
	$panel =~ s/LZOOMER/HREF=\"$realm{LnkZoom}$inline\"    TARGET=\"ZOOMER\" ACCESSKEY=\"z\" $jslink/gs;


	# Determine colour
	# Initial undefined assignment
	my $wfontcolr;

	# Loop
	foreach $text (@color)	
	{
		# Match made
		if ($text eq $color)
		{
			# Data assignment
			$wfontcolr = $cgi->popup_menu(
				-name      => 'COLOR',
				-values    => \@color,
				-default   => $color);

			# Exit loop
			last;
		}
	}

	# Assign default otherwise
	$wfontcolr = $cgi->popup_menu(
		-name      => 'COLOR',
		-values    => \@color,
		-default   => $default
	 ) unless (defined($wfontcolr));


	# Widgets
	# Assigning values
	my $wpostsend = $cgi->submit(
		-name      => $realm{'TxtSubmitButton'},
		-value     => $realm{'TxtSubmitButton'},
		-tabindex  => 3,
		-accesskey => 'p');

	# Posting window
	my $wposttext = $cgi->textarea(
		-name      => 'POST',
		-rows      => 3,
		-columns   => 50,
		-wrap      => 'SOFT',
		-tabindex  => 2,
		-accesskey => 't');

	# Popup menus
	# Character select
	my $wcharslct = $cgi->popup_menu(
		-name      => 'CHAR',
		-values    => \@char,
		-default   => $puppet,
		-tabindex  => 1);

	# Font face selct
	my $wfontface = $cgi->popup_menu(
		-name      => 'FACE',
		-values    => \@face,
		-default   => $face);

	# Fonr size select
	my $wfontsize = $cgi->popup_menu(
		-name      => 'SIZE',
		-values    => \@size,
		-default   => $size);


	# Scroll back functionality
	my $wscrladvd = ($choice < 1) ? $cgi->textfield('SCROLLBACK', '', 3, 3) . " " . $realm{'TxtScrollback'} : '&nbsp;';
	my $wscrlback = ($choice < 1) ? $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'}) : '&nbsp;';
	my $wscrlctrl = ($choice < 1) ? $cgi->checkbox(-name=>'GETPOSTS',   -label=>$realm{'TxtRemote'})     : '&nbsp;';
	my $wstrmctrl = ($choice > 1) ? $cgi->checkbox(-name=>'STRMKILL',   -label=>$realm{'TxtStream'})     : '&nbsp;';


	# Locale settings
	if ($locale ne 'none')
	{
		# Bring into play
		my $wcharlocl;
		my $wcharfilt;


		# Filtering widget
		$wcharfilt = (defined($param->{'ICPOSTS'})) 
			? $cgi->checkbox(-name=>'ICPOSTS', -label=>'', -checked=>'on')
			: $cgi->checkbox(-name=>'ICPOSTS', -label=>'');

		# Appropriate cleanup
		$param->Cleanup('ICPOSTS');



		# Locale widget
		# Current location
		($locate) = $database->DataGetPuppetLocale($puppet, $param->{'USER'});


		# Strict list
		if ($locale eq 'strict')
		{
			# Location defined
			$locate = (defined($locate)) ? $locate : $realm{'TxtNone'};


			# Pull list of locales
			$database->GetListRealmLocale(\@locale, $param->{'ROOM'});

			# Append default
			unshift(@locale, $realm{'TxtNone'});

			# Generate widget
			$wcharlocl = $cgi->popup_menu('LOCL', \@locale, $locate);
		}

		# Simple textfield
		else
		{
			# Location defined
			$locate = (defined($locate)) ? $locate : '';

			# Generate widget
			$wcharlocl = $cgi->textfield('LOCL', $locate, 25, 25);
		}

		# Replace macro
		$panel =~ s/WCHARLOCL/$wcharlocl/gs;
		$panel =~ s/WCHARFILT/$wcharfilt/gs;
	}


	# Widget replacements
	# Posting widgets
	$panel =~ s/WPOSTSEND/$wpostsend/gs;
	$panel =~ s/WPOSTTEXT/$wposttext/gs;

	# Puppet selection
	$panel =~ s/WCHARSLCT/$wcharslct/gs;

	# Font selection
	$panel =~ s/WFONTCOLR/$wfontcolr/gs;
	$panel =~ s/WFONTFACE/$wfontface/gs;
	$panel =~ s/WFONTSIZE/$wfontsize/gs;

	# Scrollback/Archiving/Stream Kill
	$panel =~ s/WSCRLADVD/$wscrladvd/gs;
	$panel =~ s/WSCRLBACK/$wscrlback/gs;
	$panel =~ s/WSCRLCTRL/$wscrlctrl/gs;
	$panel =~ s/WSTRMCTRL/$wstrmctrl/gs;


	# Framed, streaming, autoscrolling 
	if ($choice != 0)
	{
		# Frame sensitive information
		$param->{'COMM'} = (defined($comm)) ? $comm : 'True';
	}


	# HTML
	# Start form
	print $cgi->start_form(-name=>'SEND');
	print $param->EmbedNormal($param->Flat()), "\n"; 


	# Display HTML
	print "$panel\n";


	# End of form
	print $cgi->end_form(), "\n";

	# Print end of html if needed
	if ($choice != 0)
	{ 
		# Internet explorer
		if ($cgi->user_agent('MSIE'))
		{
			# Display detached stream
			print $jsstrm, "\n";
		}


		# Retreive proper footer
		print $database->FetchRealmFooter($param->{'ROOM'}); 
	}
}

# Alias for dependency
sub CommDependant
{ Comm(shift, shift, 0); }

# Alias for independnt framed
sub CommIndependant
{ Comm(shift, shift, 1); }

# Alias for independant streaming
sub CommStream
{ Comm(shift, shift, 2); }

# Alias for independant auto-scrolling streaming
sub CommAutoScroll
{ Comm(shift, shift, 3); }


#####################
# DataWrite
#
# Encharged with properly handling information and actually making posts
# once formatted inserting them into the database.

sub DataWrite
{
        #####################
        # Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle

	my $database = $self->{'HANDLE'};			# Database handle
	my $locate;						# Location handler
	my $puppet;						# Puppet name and existence

	my %realm;						# Realm hash


        #####################
        # Data members

	# Assign to target if undefined
	$param->{'TARGET'} = (defined($param->{'TARGET'})) ? $param->{'TARGET'} : 'all';

	# Make sure puppet is valid
	($puppet) = $database->DataGetPuppetExistence($param->{'CHAR'}, $param->{'USER'});

	# Puppet must exist
	if (defined($puppet))
	{
		# Only do following if posts exist
		if ((defined($param->{'POST'}))
		 && ((length($param->{'POST'}) > 1)))
		{
			# Link with hash
			$database->GetHashRealm(\%realm);

			# Update if not a dice roll or system wide post
			if (($param->{'TARGET'} ne 'dice roll')
			 && ($param->{'TARGET'} ne $realm{'TxtSystem'}))
			{
				# Determine location
				# Location handling
				if (defined($param->{'LOCL'}))
				{
					# Valid entry
					if (($param->{'LOCL'} ne $realm{'TxtNone'})
					 && (length($param->{'LOCL'}) > 2))	
					{
						# Direct association
						($locate) = $param->{'LOCL'};	
					}

					# Invalid
					else
					{
						# Cleanup 
						$param->Cleanup('LOCL');
					}
				}

				# Re-assign defaults
				else
				{
					# Use defaults unless you can reset
					($locate) = $database->DataGetPuppetLocale($param->{'CHAR'}, $param->{'USER'});

					# Cleanup
					$param->Cleanup('LOCL');
				}


				# Update database entry
				$database->Quick("UPDATE Puppet 
					SET PuppetTextColour=?, 
					    PuppetTextSize=?,
					    PuppetTextFace=?, 
					    PuppetLocale=?
					WHERE PuppetName=?
					  AND PuppeteerLogin=?", 

				 $param->{'COLOR'}, 
			 	 $param->{'SIZE'},
				 $param->{'FACE'}, 
				 $locate,
				 $param->{'CHAR'},
				 $param->{'USER'});
			}

			# Handle posts
			$self->PostHandler($param, \%realm); 
		}
	
		# No post simply post for timestamp purposes
		else
		{
			# Make people aware their presence
			$database->Quick("INSERT INTO Post 
				(RealmName, 
				 PuppetName,
				 PuppeteerLogin,
				 PostTo)
				VALUES(?,?,?,'system')", 

			 $param->{'ROOM'}, 
			 $param->{'CHAR'},
			 $param->{'USER'});
		}
	}
}


#####################
# FontSelect
#
# Based on the ability to allow hosts to select which colours, fonts faces and 
# sizes they wish to allow.  This sub-routine will determin what to send
# back.  As being either the hosts entity or a complete list as default.

sub FontSelect
{
        #####################
        # Data members
        my $self     = shift;                                   # Self referencing link
        my $param    = shift;                                   # Parameter handle
	my $type     = shift;					# Font type

        my $database = $self->{'HANDLE'};                       # Database handle

	my $options;						# String of options
	my @options;						# List of options


	#####################
	# Program Area

	# Pull realm values	
	($options) = $database->DataGet("SELECT RealmFont$type 
		FROM  Realm
		WHERE RealmName=?", $param->{'ROOM'});

	# Valid or not
	unless (defined($options))
	{
		# Retreive list
		$database->GetList(\@options, "SELECT FontName
			FROM Font
			WHERE FontType=?
			ORDER BY FontCode", lc($type));
	}
	else
	{
		# Seperate into useable text
		@options = split(/:/, $options);
	}

	# Return defaults
	return @options;
}

# Alias for font color
sub FontSelectColor
{ return FontSelect(shift, shift, 'Color'); }

# Alias for font face
sub FontSelectFace
{ return FontSelect(shift, shift, 'Face'); }

# Alias for font size
sub FontSelectSize
{ return FontSelect(shift, shift, 'Size'); }


####################
# Gummy Handler
#
# This will control how a gummy is shown.   This will vary depending on access
# and which real they access at the time.  Since such behavior is for 
# moderators or supervisors only this will not be triggered often.

sub GummyHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Realm handle
	my $gummy    = shift;					# Gummy handling

	my $admin;						# Administrator	
	my $mod;						# Moderator
	my $super;						# Supervisor

	my $class;						# Gummy class
	my $choice;						# Choice of the matter


	#####################
	# Program Area

	# Only if defined
	if (defined($gummy))
	{

		# Retreive privs
		my ($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});

		# Only if supervisory or moderator
		if (($super eq 'yes') || ($mod eq 'yes'))
		{
			# Retreive access
			($choice) = $database->DataGetPuppeteerControl($param->{'ROOM'}, $param->{'USER'});

			# If in a restricted realm
			unless (defined($choice))
			{
				# Retreive gummy class
				($class) = $database->DataGetGummyClass($gummy);

				# Is gummy supervisor or moderator
				if (($class eq 'super') 
				 or ($class eq 'mod'))
				{
					# Display only default
					$gummy = $realm->{'SetGummyDefault'};
				}
			}
		}
	}
		
	# Gummy assignment
	$gummy = (defined($gummy)) ? $gummy : $realm->{'SetGummyDefault'};  

	# Return
	return $gummy;
}


#####################
# Post Handler
#
# Depending on rules will handle the formatting of the three styles of posts
# filter language and HTML if needed or specified.

sub PostHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Reference to realm

	my $cgi      = $self->{'CGI'};				# CGI handle
	my $database = $self->{'HANDLE'};			# Database handle

	my $p_handle;						# Puppet information
	my $r_handle;						# Realm information
	my $t_handle;						# Target information

	my $filter   = new eRealms::Filter();			# Filter handler
	my $option   = new eRealms::Option();			# Option handler
	my $stamp;						# Timestamp handler

	my $res;						# General results
	my $p_res;						# Puppet results
	my $r_res;						# Realm results

	my $full;						# Full formating
	my $imageless;						# No images
	my $spartan;						# Spartan formatting

	my $img_avg;						# Average shrinking
	my $img_height;						# Image height
	my $img_width;						# Image width

	my $cmd;						# Embeded post commands

	my $bio;						# Bio link
	my $class;						# Class handler
	my $dest     = 'all';					# Target
	my $gummy;						# Gummy link
	my $inline;						# Inline parameters
	my $name;						# Formatted name
	my $privs;						# Room privs
	my $tabletop;						# Table top 
	my $window;						# Window name to open

	my $image;						# Image section
	my $header;						# Combined tagline/title
	my $tag;						# Tagline
	my $title;						# Title

	my $sep;						# Between titles and posts
	my $top;						# Before titles

	my $adm;						# Administrative
	my $mod;						# Moderator
	my $sup;						# Supervisory

	my $post;						# Actual post
	my $position = 0;					# Position of post

	my $start;						# Start of table
	my $end;						# End of table

	my $target   = $param->{'TARGET'};			# Target to

	my @lang;						# Language array
	my @posts;						# Array of posts
	my @realmpic;						# Ream picture restriction


	#####################
	# Program area

	# Post commands
	$cmd = (defined($param->{'ICPOSTS'}))  ? 'play'   : 'general';
	$cmd = (defined($param->{'STRMKILL'})) ? 'stream' : $cmd;


	# Retreive puppet and realm information
	# Puppet information
	$database->Pull(\$p_handle, "SELECT 
		  PuppetName      AS \"PuppetName\",
		  PuppetPool	  AS \"PuppetPool\",
		  PuppeteerLogin  AS \"PuppeteerLogin\", 
		  PuppetTag       AS \"PuppetTag\",
		  PuppetPic       AS \"PuppetPic\",
		  PuppetPicHeight AS \"PuppetPicHeight\",
		  PuppetPicWidth  AS \"PuppetPicWidth\",
		  PuppetPicSize   AS \"PuppetPicSize\",
		  PuppetTitle     AS \"PuppetTitle\", 
		  PuppetGummy     AS \"PuppetGummy\"
		FROM  Puppet 
		WHERE PuppetName=?
		  AND PuppeteerLogin=?",

	 $param->{'CHAR'},
	 $param->{'USER'}
	);

	# Realm Information
	$database->Pull(\$r_handle, "SELECT 
		  RealmAccess   AS \"RealmAccess\",
		  RealmLanguage AS \"RealmLanguage\", 
		  RealmFilter   AS \"RealmFilter\",
		  RealmTags     AS \"RealmTags\",
		  RealmDateLoc  AS \"RealmDateLoc\"
		FROM  Realm 
		WHERE RealmName=?",

	 $param->{'ROOM'}
	);

	# Associate to hash
	$p_res = $p_handle->fetchrow_hashref();
	$r_res = $r_handle->fetchrow_hashref(); 	


	# Frame border and class
	# General posts
	if ($target eq 'all')
	{ 
		$class = 'general';
		$tabletop = "CLASS=\"PSTNRM\""; 
	}

	# System level posts
	elsif ($target eq $realm->{'TxtSystem'})
	{ 
		$class = 'system';
		$tabletop = "CLASS=\"PSTALT\">\n\t<TR CLASS=\"PSTSYS\">\n\t\t<TD>$realm->{TagHeadSystem}</TD>\n\t<TR"; 
	}

	# Die roller
	elsif ($target eq 'dice roll')
	{ 
		# Class
		$class = 'dice';

		# HTML display
		$tabletop = "CLASS=\"PSTALT\">\n\t<TR CLASS=\"PSTDIE\">\n\t\t<TD>$realm->{TagHeadDice}</TD>\n\t<TR"; 

		# Destination
		$dest = (defined($param->{'DPVTE'})) ? $param->{'USER'} : 'all';
	}

	# Private Messages
	elsif ($target ne 'system')
	{
		# Determine class
		# Pull privs
		($sup, $adm, $mod) = $database->DataGetPrivs($param->{'USER'});

		# Assign special permissions to hosts
		$class = (($sup eq 'yes') || ($adm eq 'yes') || ($mod eq 'yes') || (defined($param->{'PRINTOUT'}))) ? 'special' : 'private';


		# HTML display
		$tabletop = "CLASS=\"PSTALT\">\n\t<TR CLASS=\"PSTPVT\">\n\t\t<TD>$realm->{TagHeadPrivate}</TD>\n\t<TR"; 

		# Set target specifically
		$dest = $param->{'TARGET'};
	}


	# Sanity checks
	$param->{'POST'} = $filter->MakeSane($param->{'POST'}, $realm->{'SetTruncate'});


	# Language filtering
	if ($r_res->{'RealmLanguage'} eq 'restricted')
	{
		# Retreive words
		$database->GetListWords(\@lang, $param->{'ROOM'});

		# Run through
		my $word;

		while ($word = shift(@lang))
		{
			# Run a global replace
			$param->{'POST'} =~ s/$word/$r_res->{'RealmFilter'}/gsi;
			$param->{'CHAR'} =~ s/$word/$r_res->{'RealmFilter'}/gsi;
		}
	}

	# Create instance
	$stamp = new eRealms::Stamp($database, $param, $realm, $r_res->{'RealmDateLoc'}, $target);


	# Bio link
	# Determin gummy
	$gummy = $self->GummyHandler($database, $param, $realm, $p_res->{'PuppetGummy'});

	# Inline parameters
	$inline = $param->Crypt(BCHAR=>$param->{'CHAR'}, BPOOL=>$p_res->{'PuppetPool'});

	# Target
	$window = uc($param->{'CHAR'});
	$window =~ s/\W//gs;

	# Generate bio
	$bio = ($gummy ne 'NIL') ? "<A HREF=\"$realm->{'LnkBio'}$inline\" TARGET=\"$window\"><IMG SRC=\"$gummy\" BORDER=\"0\" ALIGN=\"LEFT\"></A>" : "";
	$bio = $stamp->Top($bio);
	$bio = $stamp->Removed($bio);


	# Format title
	if (defined($p_res->{'PuppetTitle'}))
	{ 
		# Define naming
		$name = ($gummy ne 'NIL') ? $param->{'CHAR'} : "<A HREF=\"$realm->{'LnkBio'}$inline\" TARGET=\"_blank\">$param->{CHAR}</A>";


		# Verify if NAME attribute exists
		if ($p_res->{'PuppetTitle'} =~ /NAME/)
		{
			# Prevent tag embed
			$p_res->{'PuppetTitle'} =~ s/<NAME>/NAME/g;

			# Replace with puppet name
			$p_res->{'PuppetTitle'} =~ s/NAME/$name/g;
			$title = $p_res->{'PuppetTitle'};
		}

		# Otherwise prepend name
		else
		{ $title = "$name $p_res->{PuppetTitle}"; }
	}
	

	# Assign chacter name to title
	else
	{ $title = $param->{'CHAR'}; }	

	# Create formatted line
	$title = "$bio <FONT CLASS=\"TITLE\">$title</FONT>";


	# Tagline handling
	$tag = (defined($p_res->{'PuppetTag'})) ? "(<FONT CLASS=\"TAGLINE\">$puppet$p_res->{'PuppetTag'}</FONT>)" : "";
	$tag = $stamp->Bottom($tag);


	# Determine font colour, face and size
	my ($fontc) = $database->DataGetFontCode($param->{'COLOR'}, 'color');
	my ($fontf) = $database->DataGetFontCode($param->{'FACE'}, 'face');
	my ($fonts) = $database->DataGetFontCode($param->{'SIZE'}, 'size');

	# Image handling
	$image = (defined($p_res->{'PuppetPic'})) ?  "<A HREF=\"$p_res->{'PuppetPic'}\" TARGET=\"_blank\">$realm->{'TagLnkImage'}</A><BR>" : ""; 


	# Filtering of the text
	# Filter HTML if option is selected
	if ($r_res->{'RealmTags'} eq 'restricted')
	{ $param->{'POST'} =~ s/<[^>]*>//gs; }

	# Standard filtering
	else
	{ ($param->{'POST'}) = $filter->SafeHTML($param->{'POST'}); }


	# Combine title and tagline
	$header = $title . ' ' . $tag;


	# Pre-post location
	# Location specified
	if (defined($param->{'LOCL'}))
	{
		# Check for existence
		# Embed to make nice
		if ($header =~ /LOCALE/)
		{
			# Replace macro
			$header =~ s/LOCALE/$param->{'LOCL'}/gs;
		}

		# Classic behaviour
		else
		{
			# Add additional location tag
			$param->{'POST'} = "<B>~~$param->{LOCL}~~</B>\n\n$param->{POST}";
		}

		# Set class
		$class = 'play';
	}

	# Safety catch
	# No change in class
	if ($header =~ /LOCALE/)
	{
		# No location replace with none
		$header =~ s/LOCALE/$realm->{'TxtLocale'}/gs;
	}


	# Final arrangements of timestamp
	$header = $stamp->Header($header);



	# Large post handling
	# Divide and conquer
	if ((length($param->{'POST'})) > 10240)
	{
		# Cannot allow stray HTML tags
		$param->{'POST'} =~ s/<[^>]*>//gs;

		# Prepend beginning of posts
		$post = substr($param->{'POST'}, 0, 10240);
		push(@posts, "$post ...");

		$position++;

		# Truncate into edible bites
		while (length(substr($param->{'POST'}, ($position * 10240), 10241)) > 10240)
		{
			$post = substr($param->{'POST'}, ($position * 10240), 10240);
			push(@posts, "... $post ...");

			$position++;
		}

		# Append whatever is left
		$post = substr($param->{'POST'}, ($position * 10240), 10240);
		push(@posts, "... $post");
	}
	# Pass as first element of array
	else
	{ push(@posts, "$param->{'POST'}"); }


	# Clear values
	$param->Cleanup('POST', 'STRMKILL');


	# Preformat seperator
	$top = "<!-HEAD-->";
	$sep = "<!-POST--><BR><BR>";

	# Cycle though individual sections
	foreach $post (@posts)
	{

		# Handling of links
		$post      =~ s/$/\n/;
		$post      =~ s/(http:\/\/.*?)(\s|$)/<A HREF="$1" TARGET="_blank">$realm->{TagHyperlink}<\/A> /gm;
		$post      =~ s/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/<A HREF="mailto:$1">$realm->{TagEmail}<\/A> /gm;

		# Add colors
		$post = "<FONT SIZE=\"$fonts\" COLOR=\"$fontc\" FACE=\"$fontf\">$post</FONT>";

		# Full formatting
		if (defined($p_res->{'PuppetPic'}))
		{
			# Retreive image size information
			my ($p_height, $p_width, $p_size) = $database->DataGetRealmImageInfo($param->{'ROOM'});

			# Format image format accordingly
			if (($p_size != 0) && ($p_res->{'PuppetPicSize'} > $p_size)) 
			{
				# Imageless format 
				$full = "$image$top$header$sep$post"; 
			}
			else
			{
				# Define images
				$img_height = $p_res->{'PuppetPicHeight'};
				$img_width  = $p_res->{'PuppetPicWidth'};

				# Ignore non restricted realms
				if (($p_height != 0) && ($p_width != 0))
				{
					# Height handling
					if ($img_height > $p_height)
					{
						# Get average
						$img_avg = $p_height / $img_height;

						# Forceful resize
						$img_height = int($img_height * $img_avg);
						$img_width  = int($img_width  * $img_avg);
					}

					# Width handling
					if ($img_width > $p_width)
					{
						# Get average
						$img_avg = $p_width / $img_width;

						# Forceful resize
						$img_height = int($img_height * $img_avg);
						$img_width  = int($img_width  * $img_avg);
					}
				}

				# Generate link
 				$full = "<IMG SRC=\"$p_res->{PuppetPic}\" HEIGHT=\"$img_height\" WIDTH=\"$img_width\" ALIGN=\"left\" BORDER=\"0\">$top$header$sep$post"; 
			}
		}
		else
		{ $full = "$top$header$sep$post"; }
	

		# Format non imaged format
		$imageless = "$image$top$header$sep$post";


		# Create spartan format
		$post =~ s/<[^>]*>//gs;
		$spartan = "$image$bio$top <FONT CLASS=\"TITLE\">$param->{'CHAR'}</FONT> ($stamp->{'DATE'})$sep$post<BR><BR>";


		# Embed all formats in tables
		$start = "<BR>\n<TABLE $tabletop>\n\t<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";

		# Convert \n to a useable character	
		$full      =~ s/\n/<BR>/gs;
		$imageless =~ s/\n/<BR>/gs;
		$spartan   =~ s/\n/<BR>/gs;

		# Assembly
		$full      = $start . $full      . $end;
		$imageless = $start . $imageless . $end;
		$spartan   = $start . $spartan   . $end;


		# Normal messages
		if (($target eq 'all')
		 || ($target eq 'dice roll'))
		{
			# Insert into database
			$database->Quick("INSERT INTO Post
				(RealmName,
				 PuppetName,
				 PuppeteerLogin,
				 PostTo,
				 PostClass,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat,
				 PostCmd)		 
				VALUES(?,?,?,?,?,?,?,?,?)",
	
			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'USER'}, 
			 $dest, 
			 $class,
			 $full, 
			 $imageless, 
			 $spartan,
			 $cmd);
		}

		# System wide
		elsif ($target eq $realm->{'TxtSystem'})
		{
			# Prepare and execute
			# Design
			$database->Pull(\$t_handle, "SELECT RealmName
				FROM Post 
				WHERE PostTimestamp > getInterval('10 Minutes')
				GROUP BY RealmName");


			# Run through each realm
			while (($res) = $t_handle->fetchrow())
			{
				# Insert into database
				$database->Quick("INSERT INTO Post 
					(RealmName,
					 PuppetName,
					 PuppeteerLogin,
					 PostTo,
					 PostClass,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat,
					 PostCmd)
					VALUES(?,?,?,?,?,?,?,?,?)", 
				 $res, 
				 $param->{'CHAR'}, 
				 $param->{'USER'}, 
				 $dest,
				 $class,
				 $full, 
				 $imageless, 
				 $spartan,
				 $cmd);
			}

			# End statement
			$t_handle->finish();
		}

		# Insert message to be returned to poster
		elsif ($param->{'TARGET'} ne 'system')
		{
			# Send to target
			$database->Quick("INSERT INTO Post
				(RealmName,
				 PuppetName,
				 PuppeteerLogin,
				 PostTo,
				 PostClass,
				 PostFullFormat,
				 PostImagelessFormat,
				 PostPartialFormat,
				 PostCmd)
				VALUES(?,?,?,?,?,?,?,?,?)", 

			 $param->{'ROOM'}, 
			 $param->{'CHAR'}, 
			 $param->{'USER'}, 
			 $dest,
			 $class,
			 $full, 
			 $imageless, 
			 $spartan,
			 $cmd);


			# No doubling for self posts
			if ($dest ne $param->{'SOURCE'})
			{
				# Reset target
				$dest  = $param->{'SOURCE'};
				$class = 'duplicate';

				# Send back owner
				$database->Quick("INSERT INTO Post
					(RealmName,
					 PuppetName,
					 PuppeteerLogin,
					 PostTo,
					 PostClass,
					 PostFullFormat,
					 PostImagelessFormat,
					 PostPartialFormat,
					 PostCmd)
					VALUES(?,?,?,?,?,?,?,?,?)",

				 $param->{'ROOM'}, 
				 $param->{'CHAR'}, 
				 $param->{'USER'}, 
				 $dest, 
				 $class,
				 $full, 
				 $imageless, 
				 $spartan,
				 $cmd);
			}
		}
	}


	# End of handles
	$r_handle->finish();
	$p_handle->finish();
}

1;								# Return true
